@@ -28,6 +28,7 @@
 
 #include "gstglelements.h"
 #include "gstgluploadelement.h"
+#include "gstglutils.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_gl_upload_element_debug);
 #define GST_CAT_DEFAULT gst_gl_upload_element_debug
@@ -60,6 +61,9 @@ static GstFlowReturn gst_gl_upload_element_transform (GstBaseTransform * bt,
 static gboolean gst_gl_upload_element_stop (GstBaseTransform * bt);
 static GstCaps *gst_gl_upload_element_fixate_caps (GstBaseTransform * bt,
     GstPadDirection direction, GstCaps * caps, GstCaps * othercaps);
+static gboolean
+gst_gl_upload_element_transform_meta (GstBaseTransform * bt,
+    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf);
 static GstStateChangeReturn
 gst_gl_upload_element_change_state (GstElement * element,
     GstStateChange transition);
@@ -112,6 +116,7 @@ gst_gl_upload_element_class_init (GstGLUploadElementClass * klass)
   bt_class->transform = gst_gl_upload_element_transform;
   bt_class->stop = gst_gl_upload_element_stop;
   bt_class->fixate_caps = gst_gl_upload_element_fixate_caps;
+  bt_class->transform_meta = gst_gl_upload_element_transform_meta;
 
   element_class->change_state = gst_gl_upload_element_change_state;
 
@@ -333,6 +338,32 @@ gst_gl_upload_element_transform (GstBaseTransform * bt, GstBuffer * buffer,
   return GST_FLOW_OK;
 }
 
+static gboolean
+gst_gl_upload_element_transform_meta (GstBaseTransform * bt,
+    GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf)
+{
+  const GstMetaInfo *info = meta->info;
+  gboolean should_copy = TRUE;
+  const gchar *valid_tags[] = {
+    GST_META_TAG_VIDEO_STR,
+    GST_META_TAG_VIDEO_ORIENTATION_STR,
+    GST_META_TAG_VIDEO_SIZE_STR,
+    GST_META_TAG_VIDEO_COLORSPACE_STR,
+    NULL
+  };
+
+  should_copy = gst_meta_api_type_tags_contain_only (info->api, valid_tags);
+
+  /* Can't handle the tags in this meta, let the parent class handle it */
+  if (!should_copy) {
+    return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (bt,
+        outbuf, meta, inbuf);
+  }
+
+  /* No need to transform, we can safely copy this meta */
+  return TRUE;
+}
+
 static GstCaps *
 gst_gl_upload_element_fixate_caps (GstBaseTransform * bt,
     GstPadDirection direction, GstCaps * caps, GstCaps * othercaps)
