@@ -20,6 +20,7 @@
 
 /**
  * SECTION:gstfdmemory
+ * @title: GstFdAllocator
  * @short_description: Memory wrapper for fd backed memory
  * @see_also: #GstMemory
  *
@@ -69,7 +70,7 @@ gst_fd_mem_free (GstAllocator * allocator, GstMemory * gmem)
       && !(mem->flags & GST_FD_MEMORY_FLAG_DONT_CLOSE))
     close (mem->fd);
   g_mutex_clear (&mem->lock);
-  g_slice_free (GstFdMemory, mem);
+  g_free (mem);
   GST_DEBUG ("%p: freed", mem);
 #endif
 }
@@ -96,6 +97,12 @@ gst_fd_mem_map (GstMemory * gmem, gsize maxsize, GstMapFlags flags)
     if ((mem->mmapping_flags & prot) == prot) {
       ret = mem->data;
       mem->mmap_count++;
+    } else if ((mem->flags & GST_FD_MEMORY_FLAG_KEEP_MAPPED)
+        && mem->mmap_count == 0
+        && mprotect (mem->data, gmem->maxsize, prot) == 0) {
+      ret = mem->data;
+      mem->mmapping_flags = prot;
+      mem->mmap_count++;
     }
 
     goto out;
@@ -153,8 +160,12 @@ gst_fd_mem_unmap (GstMemory * gmem)
   if (gmem->parent)
     return gst_fd_mem_unmap (gmem->parent);
 
-  if (mem->flags & GST_FD_MEMORY_FLAG_KEEP_MAPPED)
+  if (mem->flags & GST_FD_MEMORY_FLAG_KEEP_MAPPED) {
+    g_mutex_lock (&mem->lock);
+    mem->mmap_count--;
+    g_mutex_unlock (&mem->lock);
     return;
+  }
 
   g_mutex_lock (&mem->lock);
   if (mem->data && !(--mem->mmap_count)) {
@@ -185,7 +196,7 @@ gst_fd_mem_share (GstMemory * gmem, gssize offset, gssize size)
   if (size == -1)
     size = gmem->maxsize - offset;
 
-  sub = g_slice_new0 (GstFdMemory);
+  sub = g_new0 (GstFdMemory, 1);
   /* the shared memory is always readonly */
   gst_memory_init (GST_MEMORY_CAST (sub), GST_MINI_OBJECT_FLAGS (parent) |
       GST_MINI_OBJECT_FLAG_LOCK_READONLY, mem->mem.allocator, parent,
@@ -235,16 +246,20 @@ gst_fd_allocator_init (GstFdAllocator * allocator)
  *
  * Return a new fd allocator.
  *
- * Returns: (transfer full): a new fd allocator, or NULL if the allocator
- *    isn't available. Use gst_object_unref() to release the allocator after
- *    usage
+ * Returns: (transfer full): a new fd allocator. Use gst_object_unref() to
+ * release the allocator after usage
  *
  * Since: 1.6
  */
 GstAllocator *
 gst_fd_allocator_new (void)
 {
-  return g_object_new (GST_TYPE_FD_ALLOCATOR, NULL);
+  GstAllocator *alloc;
+
+  alloc = g_object_new (GST_TYPE_FD_ALLOCATOR, NULL);
+  gst_object_ref_sink (alloc);
+
+  return alloc;
 }
 
 /**
@@ -256,10 +271,10 @@ gst_fd_allocator_new (void)
  *
  * Return a %GstMemory that wraps a generic file descriptor.
  *
- * Returns: (transfer full): a GstMemory based on @allocator.
+ * Returns: (transfer full) (nullable): a GstMemory based on @allocator.
  * When the buffer will be released the allocator will close the @fd unless
  * the %GST_FD_MEMORY_FLAG_DONT_CLOSE flag is specified.
- * The memory is only mmapped on gst_buffer_mmap() request.
+ * The memory is only mmapped on gst_buffer_map() request.
  *
  * Since: 1.6
  */
@@ -272,7 +287,7 @@ gst_fd_allocator_alloc (GstAllocator * allocator, gint fd, gsize size,
 
   g_return_val_if_fail (GST_IS_FD_ALLOCATOR (allocator), NULL);
 
-  mem = g_slice_new0 (GstFdMemory);
+  mem = g_new0 (GstFdMemory, 1);
   gst_memory_init (GST_MEMORY_CAST (mem), 0, GST_ALLOCATOR_CAST (allocator),
       NULL, size, 0, 0, size);
 
