@@ -182,6 +182,52 @@ check_valid_channel_positions (const GstAudioChannelPosition * position,
   return TRUE;
 }
 
+/**
+ * gst_audio_reorder_channels_with_reorder_map:
+ * @data: (array length=size) (element-type guint8): The pointer to
+ *   the memory.
+ * @size: The size of the memory.
+ * @bps: The number of bytes per sample.
+ * @channels: The number of channels.
+ * @reorder_map: (array length=channels): The channel reorder map.
+ *
+ * Reorders @data with the given @reorder_map.
+ *
+ * The reorder map can be retrieved for example with
+ * gst_audio_get_channel_reorder_map().
+ *
+ * Note: this function assumes the audio data is in interleaved layout
+ *
+ * Since: 1.26
+ */
+void
+gst_audio_reorder_channels_with_reorder_map (gpointer data, gsize size,
+    gint bps, gint channels, const gint * reorder_map)
+{
+  gint bpf = bps * channels;
+  guint8 *ptr = data;
+  gsize n;
+
+  g_return_if_fail (data != NULL);
+  g_return_if_fail (size % (bps * channels) == 0);
+  g_return_if_fail (bps > 0);
+  g_return_if_fail (bps <= 64);
+  g_return_if_fail (channels > 0);
+  g_return_if_fail (channels <= 64);
+  g_return_if_fail (reorder_map != NULL);
+
+  n = size / bpf;
+  for (gsize i = 0; i < n; i++) {
+    guint8 tmp[64 * 8];
+
+    memcpy (tmp, ptr, bpf);
+    for (gint j = 0; j < channels; j++)
+      memcpy (ptr + reorder_map[j] * bps, tmp + j * bps, bps);
+
+    ptr += bpf;
+  }
+}
+
 /**
  * gst_audio_reorder_channels:
  * @data: (array length=size) (element-type guint8): The pointer to
@@ -189,13 +235,21 @@ check_valid_channel_positions (const GstAudioChannelPosition * position,
  * @size: The size of the memory.
  * @format: The %GstAudioFormat of the buffer.
  * @channels: The number of channels.
- * @from: (array): The channel positions in the buffer.
- * @to: (array): The channel positions to convert to.
+ * @from: (array length=channels): The channel positions in the buffer.
+ * @to: (array length=channels): The channel positions to convert to.
  *
  * Reorders @data from the channel positions @from to the channel
  * positions @to. @from and @to must contain the same number of
  * positions and the same positions, only in a different order.
  *
+ * This function internally calls gst_audio_get_channel_reorder_map() and
+ * gst_audio_reorder_channels_with_reorder_map(). It is more efficient to call
+ * gst_audio_get_channel_reorder_map() once to retrieve the reorder map and
+ * then call gst_audio_reorder_channels_with_reorder_map() with the same
+ * reorder map until the channel positions change.
+ *
+ * Note: this function assumes the audio data is in interleaved layout
+ *
  * Returns: %TRUE if the reordering was possible.
  */
 gboolean
@@ -204,11 +258,7 @@ gst_audio_reorder_channels (gpointer data, gsize size, GstAudioFormat format,
     const GstAudioChannelPosition * to)
 {
   const GstAudioFormatInfo *info;
-  gint i, j, n;
   gint reorder_map[64] = { 0, };
-  guint8 *ptr;
-  gint bpf, bps;
-  guint8 tmp[64 * 8];
 
   info = gst_audio_format_get_info (format);
 
@@ -220,7 +270,6 @@ gst_audio_reorder_channels (gpointer data, gsize size, GstAudioFormat format,
   g_return_val_if_fail (info->width <= 8 * 64, FALSE);
   g_return_val_if_fail (size % ((info->width * channels) / 8) == 0, FALSE);
   g_return_val_if_fail (channels > 0, FALSE);
-  g_return_val_if_fail (channels <= 64, FALSE);
 
   if (size == 0)
     return TRUE;
@@ -228,21 +277,37 @@ gst_audio_reorder_channels (gpointer data, gsize size, GstAudioFormat format,
   if (gst_audio_channel_positions_equal (from, to, channels))
     return TRUE;
 
+  g_return_val_if_fail (channels <= 64, FALSE);
+
   if (!gst_audio_get_channel_reorder_map (channels, from, to, reorder_map))
     return FALSE;
 
-  bps = info->width / 8;
-  bpf = bps * channels;
-  ptr = data;
+  gst_audio_reorder_channels_with_reorder_map (data, size, info->width / 8,
+      channels, reorder_map);
 
-  n = size / bpf;
-  for (i = 0; i < n; i++) {
+  return TRUE;
+}
 
-    memcpy (tmp, ptr, bpf);
-    for (j = 0; j < channels; j++)
-      memcpy (ptr + reorder_map[j] * bps, tmp + j * bps, bps);
+static gboolean
+gst_audio_meta_reorder_channels (GstAudioMeta * meta,
+    const GstAudioChannelPosition * from, const GstAudioChannelPosition * to)
+{
+  gint reorder_map[64] = { 0, };
+  gsize tmp_offsets[64] = { 0, };
+  gint i;
 
-    ptr += bpf;
+  g_return_val_if_fail (meta, FALSE);
+  g_return_val_if_fail (meta->info.channels > 0, FALSE);
+  g_return_val_if_fail (meta->info.channels <= 64, FALSE);
+  g_return_val_if_fail (meta->offsets != NULL, FALSE);
+
+  if (!gst_audio_get_channel_reorder_map (meta->info.channels, from, to,
+          reorder_map))
+    return FALSE;
+
+  memcpy (tmp_offsets, meta->offsets, meta->info.channels * sizeof (gsize));
+  for (i = 0; i < meta->info.channels; i++) {
+    meta->offsets[reorder_map[i]] = tmp_offsets[i];
   }
 
   return TRUE;
@@ -253,8 +318,8 @@ gst_audio_reorder_channels (gpointer data, gsize size, GstAudioFormat format,
  * @buffer: The buffer to reorder.
  * @format: The %GstAudioFormat of the buffer.
  * @channels: The number of channels.
- * @from: (array): The channel positions in the buffer.
- * @to: (array): The channel positions to convert to.
+ * @from: (array length=channels): The channel positions in the buffer.
+ * @to: (array length=channels): The channel positions to convert to.
  *
  * Reorders @buffer from the channel positions @from to the channel
  * positions @to. @from and @to must contain the same number of
@@ -269,7 +334,8 @@ gst_audio_buffer_reorder_channels (GstBuffer * buffer,
     const GstAudioChannelPosition * from, const GstAudioChannelPosition * to)
 {
   GstMapInfo info;
-  gboolean ret;
+  GstAudioMeta *meta;
+  gboolean ret = TRUE;
 
   g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
   g_return_val_if_fail (gst_buffer_is_writable (buffer), FALSE);
@@ -277,15 +343,20 @@ gst_audio_buffer_reorder_channels (GstBuffer * buffer,
   if (gst_audio_channel_positions_equal (from, to, channels))
     return TRUE;
 
-  if (!gst_buffer_map (buffer, &info, GST_MAP_READWRITE))
-    return FALSE;
+  meta = gst_buffer_get_audio_meta (buffer);
+  if (meta && meta->info.layout == GST_AUDIO_LAYOUT_NON_INTERLEAVED) {
+    g_return_val_if_fail (channels == meta->info.channels, FALSE);
 
-  ret =
-      gst_audio_reorder_channels (info.data, info.size, format, channels, from,
-      to);
+    ret = gst_audio_meta_reorder_channels (meta, from, to);
+  } else {
+    if (!gst_buffer_map (buffer, &info, GST_MAP_READWRITE))
+      return FALSE;
 
-  gst_buffer_unmap (buffer, &info);
+    ret = gst_audio_reorder_channels (info.data, info.size, format, channels,
+        from, to);
 
+    gst_buffer_unmap (buffer, &info);
+  }
   return ret;
 }
 
@@ -396,9 +467,9 @@ no_channel_mask:
 /**
  * gst_audio_get_channel_reorder_map:
  * @channels: The number of channels.
- * @from: (array): The channel positions to reorder from.
- * @to: (array): The channel positions to reorder to.
- * @reorder_map: (array): Pointer to the reorder map.
+ * @from: (array length=channels): The channel positions to reorder from.
+ * @to: (array length=channels): The channel positions to reorder to.
+ * @reorder_map: (array length=channels): Pointer to the reorder map.
  *
  * Returns a reorder map for @from to @to that can be used in
  * custom channel reordering code, e.g. to convert from or to the
@@ -421,6 +492,7 @@ gst_audio_get_channel_reorder_map (gint channels,
 
   g_return_val_if_fail (reorder_map != NULL, FALSE);
   g_return_val_if_fail (channels > 0, FALSE);
+  g_return_val_if_fail (channels <= 64, FALSE);
   g_return_val_if_fail (from != NULL, FALSE);
   g_return_val_if_fail (to != NULL, FALSE);
   g_return_val_if_fail (check_valid_channel_positions (from, channels, FALSE,
@@ -642,7 +714,7 @@ position_to_string (GstAudioChannelPosition pos)
  * Returns: (transfer full): a newly allocated string representing
  * @position
  *
- * Since 1.10
+ * Since: 1.10
  */
 gchar *
 gst_audio_channel_positions_to_string (const GstAudioChannelPosition * position,
