@@ -86,6 +86,8 @@ gst_audio_stream_align_new (gint rate, GstClockTime alignment_threshold,
   GstAudioStreamAlign *align;
 
   g_return_val_if_fail (rate != 0, NULL);
+  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (alignment_threshold), NULL);
+  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (discont_wait), NULL);
 
   align = g_new0 (GstAudioStreamAlign, 1);
   align->rate = rate;
@@ -172,7 +174,7 @@ gst_audio_stream_align_set_rate (GstAudioStreamAlign * align, gint rate)
  * Since: 1.14
  */
 gint
-gst_audio_stream_align_get_rate (GstAudioStreamAlign * align)
+gst_audio_stream_align_get_rate (const GstAudioStreamAlign * align)
 {
   g_return_val_if_fail (align != NULL, 0);
 
@@ -193,6 +195,7 @@ gst_audio_stream_align_set_alignment_threshold (GstAudioStreamAlign *
     align, GstClockTime alignment_threshold)
 {
   g_return_if_fail (align != NULL);
+  g_return_if_fail (GST_CLOCK_TIME_IS_VALID (alignment_threshold));
 
   align->alignment_threshold = alignment_threshold;
 }
@@ -208,7 +211,8 @@ gst_audio_stream_align_set_alignment_threshold (GstAudioStreamAlign *
  * Since: 1.14
  */
 GstClockTime
-gst_audio_stream_align_get_alignment_threshold (GstAudioStreamAlign * align)
+gst_audio_stream_align_get_alignment_threshold (const GstAudioStreamAlign *
+    align)
 {
   g_return_val_if_fail (align != NULL, 0);
 
@@ -229,6 +233,7 @@ gst_audio_stream_align_set_discont_wait (GstAudioStreamAlign * align,
     GstClockTime discont_wait)
 {
   g_return_if_fail (align != NULL);
+  g_return_if_fail (GST_CLOCK_TIME_IS_VALID (discont_wait));
 
   align->discont_wait = discont_wait;
 }
@@ -244,7 +249,7 @@ gst_audio_stream_align_set_discont_wait (GstAudioStreamAlign * align,
  * Since: 1.14
  */
 GstClockTime
-gst_audio_stream_align_get_discont_wait (GstAudioStreamAlign * align)
+gst_audio_stream_align_get_discont_wait (const GstAudioStreamAlign * align)
 {
   g_return_val_if_fail (align != NULL, 0);
 
@@ -280,7 +285,8 @@ gst_audio_stream_align_mark_discont (GstAudioStreamAlign * align)
  * Since: 1.14
  */
 GstClockTime
-gst_audio_stream_align_get_timestamp_at_discont (GstAudioStreamAlign * align)
+gst_audio_stream_align_get_timestamp_at_discont (const GstAudioStreamAlign *
+    align)
 {
   g_return_val_if_fail (align != NULL, GST_CLOCK_TIME_NONE);
 
@@ -299,7 +305,8 @@ gst_audio_stream_align_get_timestamp_at_discont (GstAudioStreamAlign * align)
  * Since: 1.14
  */
 guint64
-gst_audio_stream_align_get_samples_since_discont (GstAudioStreamAlign * align)
+gst_audio_stream_align_get_samples_since_discont (const GstAudioStreamAlign *
+    align)
 {
   g_return_val_if_fail (align != NULL, 0);
 
@@ -363,6 +370,7 @@ gst_audio_stream_align_process (GstAudioStreamAlign * align,
     discont = TRUE;
   } else {
     guint64 diff, max_sample_diff;
+    GstClockTime expected_time;
 
     /* Check discont */
     if (align->rate > 0) {
@@ -371,15 +379,26 @@ gst_audio_stream_align_process (GstAudioStreamAlign * align,
       diff = ABSDIFF (end_offset, align->next_offset);
     }
 
-    max_sample_diff =
+    expected_time =
+        gst_util_uint64_scale (align->next_offset, GST_SECOND,
+        ABS (align->rate));
+
+    max_sample_diff = MAX (1,
         gst_util_uint64_scale_int (align->alignment_threshold,
-        ABS (align->rate), GST_SECOND);
+            ABS (align->rate), GST_SECOND));
 
     /* Discont! */
     if (G_UNLIKELY (diff >= max_sample_diff)) {
       if (align->discont_wait > 0) {
         if (align->discont_time == GST_CLOCK_TIME_NONE) {
-          align->discont_time = align->rate > 0 ? start_time : end_time;
+          if (align->rate > 0
+              && ABSDIFF (expected_time, start_time) >= align->discont_wait)
+            discont = TRUE;
+          else if (align->rate < 0
+              && ABSDIFF (expected_time, end_time) >= align->discont_wait)
+            discont = TRUE;
+          else
+            align->discont_time = expected_time;
         } else if ((align->rate > 0
                 && ABSDIFF (start_time,
                     align->discont_time) >= align->discont_wait)
