@@ -52,8 +52,6 @@ struct _GstGLBaseSrcPrivate
 
   GstGLMemory *out_tex;
   gint64 timestamp_offset;      /* base offset */
-  gint64 accum_rtime;
-  gint64 accum_frames;
   gint64 n_frames;              /* total frames sent */
   gboolean negotiated;
   gboolean gl_result;
@@ -212,12 +210,6 @@ gst_gl_base_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
 
   glbasesrc->priv->negotiated = TRUE;
 
-  glbasesrc->priv->accum_rtime += glbasesrc->running_time;
-  glbasesrc->priv->accum_frames += glbasesrc->priv->n_frames;
-
-  glbasesrc->running_time = 0;
-  glbasesrc->priv->n_frames = 0;
-
   gst_caps_replace (&glbasesrc->out_caps, caps);
 
   return TRUE;
@@ -430,11 +422,10 @@ gst_gl_base_src_fill (GstPushSrc * psrc, GstBuffer * buffer)
   g_rec_mutex_unlock (&src->priv->context_lock);
 
   GST_BUFFER_TIMESTAMP (buffer) =
-      src->priv->accum_rtime + src->priv->timestamp_offset + src->running_time;
-  GST_BUFFER_OFFSET (buffer) = src->priv->accum_frames + src->priv->n_frames;
+      src->priv->timestamp_offset + src->running_time;
+  GST_BUFFER_OFFSET (buffer) = src->priv->n_frames;
   src->priv->n_frames++;
-  GST_BUFFER_OFFSET_END (buffer) =
-      src->priv->accum_frames + src->priv->n_frames;
+  GST_BUFFER_OFFSET_END (buffer) = src->priv->n_frames;
   if (src->out_info.fps_n) {
     next_time = gst_util_uint64_scale_int (src->priv->n_frames * GST_SECOND,
         src->out_info.fps_d, src->out_info.fps_n);
@@ -696,8 +687,8 @@ gst_gl_base_src_change_state (GstElement * element, GstStateChange transition)
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   GST_DEBUG_OBJECT (src, "changing state: %s => %s",
-      gst_state_get_name (GST_STATE_TRANSITION_CURRENT (transition)),
-      gst_state_get_name (GST_STATE_TRANSITION_NEXT (transition)));
+      gst_element_state_get_name (GST_STATE_TRANSITION_CURRENT (transition)),
+      gst_element_state_get_name (GST_STATE_TRANSITION_NEXT (transition)));
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
   if (ret == GST_STATE_CHANGE_FAILURE)
@@ -729,11 +720,9 @@ gst_gl_base_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
   if (src->out_info.fps_n) {
     src->priv->n_frames = gst_util_uint64_scale (time,
         src->out_info.fps_n, src->out_info.fps_d * GST_SECOND);
-  } else {
+  } else
     src->priv->n_frames = 0;
-  }
-  src->priv->accum_frames = 0;
-  src->priv->accum_rtime = 0;
+
   if (src->out_info.fps_n) {
     src->running_time = gst_util_uint64_scale (src->priv->n_frames,
         src->out_info.fps_d * GST_SECOND, src->out_info.fps_n);
@@ -746,23 +735,3 @@ gst_gl_base_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
 
   return GST_BASE_SRC_CLASS (parent_class)->do_seek (basesrc, segment);
 }
-
-/**
- * gst_gl_base_src_get_gl_context:
- * @base_src: a #GstGLBaseSrc
- *
- * Returns: (nullable) (transfer full): the configured #GstGLContext.
- * Since: 1.28
- */
-GstGLContext *
-gst_gl_base_src_get_gl_context (GstGLBaseSrc * base_src)
-{
-  GstGLContext *ret = NULL;
-
-  g_rec_mutex_lock (&base_src->priv->context_lock);
-  if (base_src->context)
-    ret = gst_object_ref (base_src->context);
-  g_rec_mutex_unlock (&base_src->priv->context_lock);
-
-  return ret;
-}
