@@ -289,7 +289,7 @@ gst_gl_context_init (GstGLContext * context)
   context->priv = gst_gl_context_get_instance_private (context);
 
   context->window = NULL;
-  context->gl_vtable = g_slice_alloc0 (sizeof (GstGLFuncs));
+  context->gl_vtable = g_new0 (GstGLFuncs, 1);
 
   g_mutex_init (&context->priv->render_lock);
 
@@ -487,7 +487,7 @@ gst_gl_context_new_wrapped (GstGLDisplay * display, guintptr handle,
  * gst_gl_context_get_current_gl_context:
  * @context_type: a #GstGLPlatform specifying the type of context to retrieve
  *
- * Returns: The OpenGL context handle current in the calling thread or %NULL
+ * Returns: (nullable): The OpenGL context handle current in the calling thread or %NULL
  *
  * Since: 1.6
  */
@@ -536,7 +536,7 @@ gst_gl_context_get_current_gl_context (GstGLPlatform context_type)
  *
  * See also gst_gl_context_get_proc_address().
  *
- * Returns: a function pointer for @name, or %NULL
+ * Returns: (nullable): a function pointer for @name, or %NULL
  *
  * Since: 1.6
  */
@@ -727,7 +727,7 @@ gst_gl_context_finalize (GObject * object)
   }
 
   if (context->gl_vtable) {
-    g_slice_free (GstGLFuncs, context->gl_vtable);
+    g_free (context->gl_vtable);
     context->gl_vtable = NULL;
   }
 
@@ -800,7 +800,7 @@ gst_gl_context_activate (GstGLContext * context, gboolean activate)
  * gst_gl_context_get_thread:
  * @context: a #GstGLContext
  *
- * Returns: (transfer full): The #GThread, @context is current in or NULL
+ * Returns: (transfer full) (nullable): The #GThread, @context is current in or NULL
  *
  * Since: 1.6
  */
@@ -870,7 +870,7 @@ gst_gl_context_get_gl_api (GstGLContext * context)
  * void (GSTGLAPI *PFN_glGetIntegerv) (GLenum name, GLint * ret)
  * ]|
  *
- * Returns: a function pointer or %NULL
+ * Returns: (nullable): a function pointer or %NULL
  *
  * Since: 1.4
  */
@@ -901,7 +901,7 @@ gst_gl_context_get_proc_address (GstGLContext * context, const gchar * name)
  *
  * See also: gst_gl_context_get_proc_address()
  *
- * Returns: an address pointing to @name or %NULL
+ * Returns: (nullable): an address pointing to @name or %NULL
  *
  * Since: 1.4
  */
@@ -1019,8 +1019,8 @@ gst_gl_context_can_share (GstGLContext * context, GstGLContext * other_context)
 /**
  * gst_gl_context_create:
  * @context: a #GstGLContext:
- * @other_context: (allow-none): a #GstGLContext to share OpenGL objects with
- * @error: (allow-none): a #GError
+ * @other_context: (nullable): a #GstGLContext to share OpenGL objects with
+ * @error: a #GError
  *
  * Creates an OpenGL context with the specified @other_context as a context
  * to share shareable OpenGL objects with.  See the OpenGL specification for
@@ -1194,8 +1194,19 @@ _build_extension_string (GstGLContext * context)
   return ext_g_str;
 }
 
-//gboolean
-//gst_gl_context_create (GstGLContext * context, GstGLContext * other_context, GError ** error)
+typedef struct
+{
+  GError **error;
+  gboolean ret;
+} FillInfoCtx;
+
+static void
+fill_info (GstGLContext * context, gpointer data)
+{
+  FillInfoCtx *ctx = data;
+  ctx->ret = gst_gl_context_fill_info (context, ctx->error);
+}
+
 static gpointer
 gst_gl_context_create_thread (GstGLContext * context)
 {
@@ -1360,9 +1371,14 @@ gst_gl_context_create_thread (GstGLContext * context)
   g_free (display_api_s);
 
   GST_DEBUG_OBJECT (context, "Filling info");
-  if (!gst_gl_context_fill_info (context, error)) {
-    g_assert (error == NULL || *error != NULL);
-    goto failure;
+  {
+    FillInfoCtx ctx = { 0 };
+    ctx.error = error;
+    gst_gl_context_thread_add (context, fill_info, &ctx);
+    if (!ctx.ret) {
+      g_assert (error == NULL || *error != NULL);
+      goto failure;
+    }
   }
 
   context->priv->alive = TRUE;
@@ -1523,17 +1539,7 @@ gst_gl_context_fill_info (GstGLContext * context, GError ** error)
     goto failure;
   }
 
-  /* Does not implement OES_vertex_array_object properly, see
-   * https://bugzilla.gnome.org/show_bug.cgi?id=750185 */
-  if (g_strcmp0 ((const gchar *) gl->GetString (GL_VENDOR),
-          "Imagination Technologies") == 0
-      && g_strcmp0 ((const gchar *) gl->GetString (GL_RENDERER),
-          "PowerVR SGX 544MP") == 0) {
-    gl->GenVertexArrays = NULL;
-    gl->DeleteVertexArrays = NULL;
-    gl->BindVertexArray = NULL;
-    gl->IsVertexArray = NULL;
-  }
+  gst_gl_context_apply_quirks (context);
 
   if (GST_IS_GL_WRAPPED_CONTEXT (context)) {
     /* XXX: vfunc? */
@@ -1637,8 +1643,8 @@ _gst_gl_context_thread_run_generic (RunGenericData * data)
 /**
  * gst_gl_context_thread_add:
  * @context: a #GstGLContext
- * @func: (scope call): a #GstGLContextThreadFunc
- * @data: (closure): user data to call @func with
+ * @func: (scope call) (closure data): a #GstGLContextThreadFunc
+ * @data: user data to call @func with
  *
  * Execute @func in the OpenGL thread of @context with @data
  *
@@ -1670,6 +1676,9 @@ gst_gl_context_thread_add (GstGLContext * context,
 
   window = gst_gl_context_get_window (context);
 
+  GST_TRACE_OBJECT (context, "schedule function:%p data:%p", rdata.func,
+      rdata.data);
+
   gst_gl_window_send_message (window,
       GST_GL_WINDOW_CB (_gst_gl_context_thread_run_generic), &rdata);
 
@@ -1773,7 +1782,7 @@ gst_gl_context_check_feature (GstGLContext * context, const gchar * feature)
  *
  * See also gst_gl_context_activate().
  *
- * Returns: (transfer none): the #GstGLContext active in the current thread or %NULL
+ * Returns: (transfer none) (nullable): the #GstGLContext active in the current thread or %NULL
  *
  * Since: 1.6
  */
