@@ -393,8 +393,6 @@ gst_discoverer_info_finalize (GObject * object)
   if (info->toc)
     gst_toc_unref (info->toc);
 
-  g_free (info->cachefile);
-
   g_ptr_array_unref (info->missing_elements_details);
 
   G_OBJECT_CLASS (gst_discoverer_info_parent_class)->finalize (object);
@@ -475,7 +473,7 @@ gst_discoverer_info_class_init (GObjectClass * klass)
 
 /**
  * gst_discoverer_stream_info_list_free:
- * @infos: (element-type GstPbutils.DiscovererStreamInfo): a #GList of #GstDiscovererStreamInfo
+ * @infos: (transfer full) (element-type GstPbutils.DiscovererStreamInfo): a #GList of #GstDiscovererStreamInfo
  *
  * Decrements the reference count of all contained #GstDiscovererStreamInfo
  * and fress the #GList.
