@@ -22,6 +22,7 @@
 #define _GST_DISCOVERER_H_
 
 #include <gst/gst.h>
+#include <gst/pbutils/pbutils-prelude.h>
 
 G_BEGIN_DECLS
 
@@ -34,7 +35,7 @@ G_BEGIN_DECLS
 typedef struct _GstDiscovererStreamInfo GstDiscovererStreamInfo;
 typedef GObjectClass GstDiscovererStreamInfoClass;
 
-GST_EXPORT
+GST_PBUTILS_API
 GType gst_discoverer_stream_info_get_type (void);
 
 /**
@@ -59,30 +60,33 @@ GType gst_discoverer_stream_info_get_type (void);
 #define gst_discoverer_stream_info_ref(info) ((GstDiscovererStreamInfo*) g_object_ref((GObject*) info))
 #define gst_discoverer_stream_info_unref(info) (g_object_unref((GObject*) info))
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscovererStreamInfo* gst_discoverer_stream_info_get_previous(GstDiscovererStreamInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscovererStreamInfo* gst_discoverer_stream_info_get_next(GstDiscovererStreamInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstCaps*                 gst_discoverer_stream_info_get_caps(GstDiscovererStreamInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 const GstTagList*        gst_discoverer_stream_info_get_tags(GstDiscovererStreamInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 const GstToc*            gst_discoverer_stream_info_get_toc(GstDiscovererStreamInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar*             gst_discoverer_stream_info_get_stream_id(GstDiscovererStreamInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_DEPRECATED_FOR(gst_discoverer_info_get_missing_elements_installer_details)
 const GstStructure*      gst_discoverer_stream_info_get_misc(GstDiscovererStreamInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar *            gst_discoverer_stream_info_get_stream_type_nick(GstDiscovererStreamInfo* info);
 
+GST_PBUTILS_API
+gint                     gst_discoverer_stream_info_get_stream_number(GstDiscovererStreamInfo *info);
+
 /**
  * GstDiscovererContainerInfo:
  *
@@ -97,12 +101,15 @@ const gchar *            gst_discoverer_stream_info_get_stream_type_nick(GstDisc
 typedef struct _GstDiscovererContainerInfo GstDiscovererContainerInfo;
 typedef GObjectClass GstDiscovererContainerInfoClass;
 
-GST_EXPORT
+GST_PBUTILS_API
 GType gst_discoverer_container_info_get_type (void);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *gst_discoverer_container_info_get_streams(GstDiscovererContainerInfo *info);
 
+GST_PBUTILS_API
+const GstTagList* gst_discoverer_container_info_get_tags(const GstDiscovererContainerInfo *info);
+
 
 /**
  * GstDiscovererAudioInfo:
@@ -118,28 +125,28 @@ GList *gst_discoverer_container_info_get_streams(GstDiscovererContainerInfo *inf
 typedef struct _GstDiscovererAudioInfo GstDiscovererAudioInfo;
 typedef GObjectClass GstDiscovererAudioInfoClass;
 
-GST_EXPORT
+GST_PBUTILS_API
 GType gst_discoverer_audio_info_get_type (void);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint gst_discoverer_audio_info_get_channels(const GstDiscovererAudioInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint64 gst_discoverer_audio_info_get_channel_mask(const GstDiscovererAudioInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint gst_discoverer_audio_info_get_sample_rate(const GstDiscovererAudioInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint gst_discoverer_audio_info_get_depth(const GstDiscovererAudioInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint gst_discoverer_audio_info_get_bitrate(const GstDiscovererAudioInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint gst_discoverer_audio_info_get_max_bitrate(const GstDiscovererAudioInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar * gst_discoverer_audio_info_get_language(const GstDiscovererAudioInfo* info);
 
 /**
@@ -156,40 +163,40 @@ const gchar * gst_discoverer_audio_info_get_language(const GstDiscovererAudioInf
 typedef struct _GstDiscovererVideoInfo GstDiscovererVideoInfo;
 typedef GObjectClass GstDiscovererVideoInfoClass;
 
-GST_EXPORT
+GST_PBUTILS_API
 GType gst_discoverer_video_info_get_type (void);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_width(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_height(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_depth(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_framerate_num(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_framerate_denom(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_par_num(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_par_denom(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean        gst_discoverer_video_info_is_interlaced(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_bitrate(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 guint           gst_discoverer_video_info_get_max_bitrate(const GstDiscovererVideoInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean        gst_discoverer_video_info_is_image(const GstDiscovererVideoInfo* info);
 
 /**
@@ -207,10 +214,10 @@ gboolean        gst_discoverer_video_info_is_image(const GstDiscovererVideoInfo*
 typedef struct _GstDiscovererSubtitleInfo GstDiscovererSubtitleInfo;
 typedef GObjectClass GstDiscovererSubtitleInfoClass;
 
-GST_EXPORT
+GST_PBUTILS_API
 GType gst_discoverer_subtitle_info_get_type (void);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar *   gst_discoverer_subtitle_info_get_language(const GstDiscovererSubtitleInfo* info);
 
 /**
@@ -272,71 +279,71 @@ typedef struct _GstDiscovererInfo GstDiscovererInfo;
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DISCOVERER_INFO))
 typedef GObjectClass GstDiscovererInfoClass;
 
-GST_EXPORT
+GST_PBUTILS_API
 GType gst_discoverer_info_get_type (void);
 
 #define gst_discoverer_info_unref(info) (g_object_unref((GObject*)info))
 #define gst_discoverer_info_ref(info) (g_object_ref((GObject*)info))
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscovererInfo*        gst_discoverer_info_copy (GstDiscovererInfo * ptr);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar*              gst_discoverer_info_get_uri(const GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscovererResult       gst_discoverer_info_get_result(const GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscovererStreamInfo*  gst_discoverer_info_get_stream_info(GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList*                    gst_discoverer_info_get_stream_list(GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstClockTime              gst_discoverer_info_get_duration(const GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean                  gst_discoverer_info_get_seekable(const GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean                  gst_discoverer_info_get_live(const GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_DEPRECATED_FOR(gst_discoverer_info_get_missing_elements_installer_details)
 const GstStructure*       gst_discoverer_info_get_misc(const GstDiscovererInfo* info);
 
-GST_EXPORT
-const GstTagList*         gst_discoverer_info_get_tags(const GstDiscovererInfo* info); 
-GST_EXPORT
+GST_PBUTILS_DEPRECATED
+const GstTagList*         gst_discoverer_info_get_tags(const GstDiscovererInfo* info);
+GST_PBUTILS_API
 const GstToc*             gst_discoverer_info_get_toc(const GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 const gchar**             gst_discoverer_info_get_missing_elements_installer_details(const GstDiscovererInfo* info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *                   gst_discoverer_info_get_streams (GstDiscovererInfo *info,
 							   GType streamtype);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *                   gst_discoverer_info_get_audio_streams (GstDiscovererInfo *info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *                   gst_discoverer_info_get_video_streams (GstDiscovererInfo *info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *                   gst_discoverer_info_get_subtitle_streams (GstDiscovererInfo *info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GList *                   gst_discoverer_info_get_container_streams (GstDiscovererInfo *info);
 
-GST_EXPORT
+GST_PBUTILS_API
 GVariant *                gst_discoverer_info_to_variant (GstDiscovererInfo *info,
                                                           GstDiscovererSerializeFlags flags);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscovererInfo *       gst_discoverer_info_from_variant (GVariant *variant);
 
-GST_EXPORT
+GST_PBUTILS_API
 void                      gst_discoverer_stream_info_list_free (GList *infos);
 
 #define GST_TYPE_DISCOVERER \
@@ -376,66 +383,66 @@ struct _GstDiscovererClass {
   void        (*starting)        (GstDiscoverer *discoverer);
   void        (*discovered)      (GstDiscoverer *discoverer,
                                   GstDiscovererInfo *info,
-				  const GError *err);
+                                  const GError *err);
   void        (*source_setup)    (GstDiscoverer *discoverer,
                                   GstElement *source);
+  /**
+   * GstDiscovererClass::load_serialize_info:
+   * @dc: the #GstDiscoverer
+   * @uri: the uri to load the info from
+   *
+   * Loads the serialized info from the given uri.
+   *
+   * Returns: (transfer full): the #GstDiscovererInfo or %NULL if it could not be loaded
+   *
+   * Since: 1.24
+   */
+  GstDiscovererInfo *
+  (*load_serialize_info)          (GstDiscoverer *dc,
+                                   gchar *uri);
 
-  gpointer _reserved[GST_PADDING];
+  gpointer _reserved[GST_PADDING - 1];
 };
 
-GST_EXPORT
+GST_PBUTILS_API
 GType          gst_discoverer_get_type (void);
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscoverer *gst_discoverer_new (GstClockTime timeout, GError **err);
 
 /* Asynchronous API */
 
-GST_EXPORT
+GST_PBUTILS_API
 void           gst_discoverer_start (GstDiscoverer *discoverer);
 
-GST_EXPORT
+GST_PBUTILS_API
 void           gst_discoverer_stop (GstDiscoverer *discoverer);
 
-GST_EXPORT
+GST_PBUTILS_API
 gboolean       gst_discoverer_discover_uri_async (GstDiscoverer *discoverer,
 						  const gchar *uri);
 
 /* Synchronous API */
 
-GST_EXPORT
+GST_PBUTILS_API
 GstDiscovererInfo *
 gst_discoverer_discover_uri (GstDiscoverer * discoverer,
 			     const gchar * uri,
 			     GError ** err);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDiscoverer, gst_object_unref)
-#endif
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDiscovererAudioInfo, gst_object_unref)
-#endif
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDiscovererContainerInfo, gst_object_unref)
-#endif
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDiscovererInfo, gst_object_unref)
-#endif
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDiscovererStreamInfo, gst_object_unref)
-#endif
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDiscovererSubtitleInfo, gst_object_unref)
-#endif
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDiscovererVideoInfo, gst_object_unref)
-#endif
 
 G_END_DECLS
 
