@@ -108,15 +108,15 @@
  * pre-defined helper application will be called.
  *
  * The exact path of the helper application to be called is set at compile
- * time, usually by the `./configure` script based on the install prefix.
+ * time, usually by the build system based on the install prefix.
  * For a normal package build into the `/usr` prefix, this will usually
  * default to `/usr/libexec/gst-install-plugins-helper` or
  * `/usr/lib/gst-install-plugins-helper`.
  *
  * Vendors/distros who want to support GStreamer plugin installation should
- * either provide such a helper script/application or use the `./configure`
- * option `--with-install-plugins-helper=/path/to/installer` to make
- * GStreamer call an installer of their own directly.
+ * either provide such a helper script/application or use the meson option
+ * `-Dinstall_plugins_helper'=/path/to/installer` to make GStreamer call an
+ * installer of their own directly.
  *
  * It is strongly recommended that vendors provide a small helper
  * application as interlocutor to the real installer though, even more so
@@ -124,7 +124,7 @@
  * line arguments passed by GStreamer to the helper application into
  * arguments that are understood by the real installer.
  *
- * The helper application path defined at compile time can be overriden at
+ * The helper application path defined at compile time can be overridden at
  * runtime by setting the GST_INSTALL_PLUGINS_HELPER environment
  * variable. This can be useful for testing/debugging purposes.
  *
@@ -157,11 +157,10 @@
  *     libraries to use the same mechanism for their automatic plugin
  *     installation needs, or for the format to be changed should it turn
  *     out to be insufficient.
- * -   plugin system version, e.g. "0.10"
- *     This is required so that when there is a GStreamer-0.12 or
- *     GStreamer-1.0 at some point in future, the different major versions
- *     can still co-exist and use the same plugin install mechanism in the
- *     same way.
+ * -   plugin system version, e.g. "1.0"
+ *     This is required so that when there is GStreamer-2.0 at some point
+ *     in future, the different major versions can still co-exist and use
+ *     the same plugin install mechanism in the same way.
  * -   application identifier, e.g. "totem"
  *     This may also be in the form of "pid/12345" if the program name
  *     can't be obtained for some reason.
@@ -184,7 +183,7 @@
  * -   optional further fields not yet specified
  *
  * An entire ID string might then look like this, for example: `
- * gstreamer|0.10|totem|Vorbis audio decoder|decoder-audio/x-vorbis`
+ * gstreamer|1.0|totem|Vorbis audio decoder|decoder-audio/x-vorbis`
  *
  * Plugin installers parsing this ID string should expect further fields
  * also separated by '|' symbols and either ignore them, warn the user, or
@@ -217,7 +216,7 @@
  * -   1 if no appropriate installation candidate for any of the requested
  *     plugins could be found. Only return this if nothing has been
  *     installed (#GST_INSTALL_PLUGINS_NOT_FOUND)
- * -   2 if an error occured during the installation. The application will
+ * -   2 if an error occurred during the installation. The application will
  *     assume that the user will already have seen an error message by the
  *     installer in this case and will usually not show another one
  *     (#GST_INSTALL_PLUGINS_ERROR)
@@ -273,6 +272,27 @@
 
 #include <string.h>
 
+#ifndef GST_DISABLE_GST_DEBUG
+#define GST_CAT_DEFAULT gst_pb_utils_install_plugins_ensure_debug_category()
+
+static GstDebugCategory *
+gst_pb_utils_install_plugins_ensure_debug_category (void)
+{
+  static gsize cat_gonce = 0;
+
+  if (g_once_init_enter (&cat_gonce)) {
+    GstDebugCategory *cat = NULL;
+
+    GST_DEBUG_CATEGORY_INIT (cat, "install-plugins", 0,
+        "GstPbUtils plugins installation helper");
+
+    g_once_init_leave (&cat_gonce, (gsize) cat);
+  }
+
+  return (GstDebugCategory *) cat_gonce;
+}
+#endif /* GST_DISABLE_GST_DEBUG */
+
 /* best effort to make things compile and possibly even work on win32 */
 #ifndef WEXITSTATUS
 # define WEXITSTATUS(status) ((((guint)(status)) & 0xff00) >> 8)
@@ -393,13 +413,13 @@ void gst_install_plugins_context_set_startup_notification_id
  * Gtk+/Gnome application should be able to obtain the XID of the top-level
  * window like this:
  * |[
- * ##include &lt;gtk/gtk.h&gt;
+ * ##include <gtk/gtk.h>
  * ##ifdef GDK_WINDOWING_X11
- * ##include &lt;gdk/gdkx.h&gt;
+ * ##include <gdk/gdkx.h>
  * ##endif
  * ...
  * ##ifdef GDK_WINDOWING_X11
- *   xid = GDK_WINDOW_XWINDOW (GTK_WIDGET (application_window)-&gt;window);
+ *   xid = GDK_WINDOW_XWINDOW (GTK_WIDGET (application_window)->window);
  * ##endif
  * ...
  * ]|
@@ -444,7 +464,17 @@ gst_install_plugins_context_free (GstInstallPluginsContext * ctx)
   g_free (ctx);
 }
 
-static GstInstallPluginsContext *
+/**
+ * gst_install_plugins_context_copy:
+ * @ctx: a #GstInstallPluginsContext
+ *
+ * Copies a #GstInstallPluginsContext.
+ *
+ * Returns: (transfer full): A copy of @ctx
+ *
+ * Since: 1.12.1
+ */
+GstInstallPluginsContext *
 gst_install_plugins_context_copy (GstInstallPluginsContext * ctx)
 {
   GstInstallPluginsContext *ret;
@@ -603,8 +633,9 @@ gst_install_plugins_installer_exited (GPid pid, gint status, gpointer data)
  * @details: (array zero-terminated=1) (transfer none): NULL-terminated array
  *     of installer string details (see below)
  * @ctx: (allow-none): a #GstInstallPluginsContext, or NULL
- * @func: (scope async): the function to call when the installer program returns
- * @user_data: (closure): the user data to pass to @func when called, or NULL
+ * @func: (scope async) (closure user_data): the function to call when the
+ *     installer program returns
+ * @user_data: the user data to pass to @func when called, or NULL
  *
  * Requests plugin installation without blocking. Once the plugins have been
  * installed or installation has failed, @func will be called with the result
