@@ -20,19 +20,16 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include <unistd.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/check/gstcheck.h>
 #include <gst/check/gstharness.h>
-
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-#define AFORMAT "S16BE"
-#else
-#define AFORMAT "S16LE"
-#endif
+#include <gst/audio/audio.h>
 
 #define AUDIO_CAPS_STRING "audio/x-raw, " \
-                           "format = (string) " AFORMAT ", "\
+                           "format = (string) " GST_AUDIO_NE (S16) ", "\
                            "layout = (string) interleaved, " \
                            "rate = (int) 48000, " \
                            "channels = (int) 1 "
@@ -237,26 +234,27 @@ GST_START_TEST (test_opus_encode_properties)
     int value;
   } param_changes[steps] = {
     {
-    "frame-size", 40}, {
-    "inband-fec", 1}, {
-    "complexity", 5}, {
-    "bandwidth", 1104}, {
-    "frame-size", 2}, {
-    "max-payload-size", 80}, {
-    "frame-size", 60}, {
-    "max-payload-size", 900}, {
-    "complexity", 1}, {
-    "bitrate", 30000}, {
-    "frame-size", 10}, {
-    "bitrate", 300000}, {
-    "inband-fec", 0}, {
-    "frame-size", 5}, {
-    "bandwidth", 1101}, {
-    "frame-size", 10}, {
-    "bitrate", 500000}, {
-    "frame-size", 5}, {
-    "bitrate", 80000}, {
-  "complexity", 8},};
+        "frame-size", 40}, {
+        "inband-fec", 1}, {
+        "complexity", 5}, {
+        "bandwidth", 1104}, {
+        "frame-size", 2}, {
+        "max-payload-size", 80}, {
+        "frame-size", 60}, {
+        "max-payload-size", 900}, {
+        "complexity", 1}, {
+        "bitrate", 30000}, {
+        "frame-size", 10}, {
+        "bitrate", 300000}, {
+        "inband-fec", 0}, {
+        "frame-size", 5}, {
+        "bandwidth", 1101}, {
+        "frame-size", 10}, {
+        "bitrate", 500000}, {
+        "frame-size", 5}, {
+        "bitrate", 80000}, {
+        "complexity", 8},
+  };
 
   opusenc = setup_opusenc ();
 
@@ -342,6 +340,7 @@ run_getcaps_check (GstCaps * filter, GstCaps * downstream_caps,
   GstElement *capsfilter;
   GstPad *sinkpad;
   GstCaps *result;
+  gchar *caps_str;
 
   opusdec = gst_element_factory_make ("opusdec", NULL);
   capsfilter = gst_element_factory_make ("capsfilter", NULL);
@@ -352,8 +351,9 @@ run_getcaps_check (GstCaps * filter, GstCaps * downstream_caps,
     g_object_set (capsfilter, "caps", downstream_caps, NULL);
   result = gst_pad_query_caps (sinkpad, filter);
   result = remove_extra_caps_fields (result);
+  caps_str = gst_caps_to_string (result);
   fail_unless (gst_caps_is_equal (expected_result, result),
-      "Unexpected output caps: %s", gst_caps_to_string (result));
+      "Unexpected output caps: %s", caps_str);
 
   if (filter)
     gst_caps_unref (filter);
@@ -364,6 +364,7 @@ run_getcaps_check (GstCaps * filter, GstCaps * downstream_caps,
   gst_object_unref (sinkpad);
   gst_object_unref (opusdec);
   gst_object_unref (capsfilter);
+  g_free (caps_str);
 }
 
 static void
@@ -379,19 +380,19 @@ GST_START_TEST (test_opusdec_getcaps)
 {
   /* default result */
   run_getcaps_check_from_strings (NULL, NULL,
-      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,8]");
+      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,255]");
 
   /* A single supported rate downstream - should accept any upstream anyway */
   run_getcaps_check_from_strings (NULL, "audio/x-raw, rate=(int)8000",
-      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,8]");
+      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,255]");
 
   /* Two supported rates (fields as a array, not as a single int) */
   run_getcaps_check_from_strings (NULL, "audio/x-raw, rate=(int){24000, 8000}",
-      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,8]");
+      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,255]");
 
   /* One supported and one unsupported rate */
   run_getcaps_check_from_strings (NULL, "audio/x-raw, rate=(int){24000, 1000}",
-      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,8]");
+      "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)[1,255]");
 
   /* Unsupported rate */
   run_getcaps_check_from_strings (NULL, "audio/x-raw, rate=(int)1000", "EMPTY");
@@ -409,12 +410,12 @@ GST_START_TEST (test_opusdec_getcaps)
   /* Formats not acceptable */
   run_getcaps_check_from_strings ("audio/x-opus, rate=(int)1000", NULL,
       "EMPTY");
-  run_getcaps_check_from_strings ("audio/x-opus, channels=(int)200", NULL,
+  run_getcaps_check_from_strings ("audio/x-opus, channels=(int)300", NULL,
       "EMPTY");
 
   /* Should restrict the result of the caps query to the selected rate/channels */
   run_getcaps_check_from_strings ("audio/x-opus, rate=(int)8000", NULL,
-      "audio/x-opus, rate=(int)8000, channels=(int)[1,8]");
+      "audio/x-opus, rate=(int)8000, channels=(int)[1,255]");
   run_getcaps_check_from_strings ("audio/x-opus, channels=(int)2", NULL,
       "audio/x-opus, rate=(int){48000, 24000, 16000, 12000, 8000}, channels=(int)2");
 }
