@@ -22,6 +22,7 @@
 
 #include <gst/video/video.h>
 #include <gst/base/gstbasesink.h>
+#include <gst/rtp/rtp.h>
 
 #include <gst/check/gstcheck.h>
 #include <gst/check/gstharness.h>
@@ -32,17 +33,6 @@
 
 #ifndef VSCALE_TEST_GROUP
 
-static guint
-get_num_formats (void)
-{
-  guint i = 2;
-
-  while (gst_video_format_to_string ((GstVideoFormat) i) != NULL)
-    ++i;
-
-  return i;
-}
-
 static void
 check_pad_template (GstPadTemplate * tmpl)
 {
@@ -50,10 +40,9 @@ check_pad_template (GstPadTemplate * tmpl)
   GstStructure *s;
   gboolean *formats_supported;
   GstCaps *caps;
-  guint i, num_formats;
+  guint i;
 
-  num_formats = get_num_formats ();
-  formats_supported = g_new0 (gboolean, num_formats);
+  formats_supported = g_new0 (gboolean, GST_VIDEO_FORMAT_LAST);
 
   caps = gst_pad_template_get_caps (tmpl);
 
@@ -86,7 +75,7 @@ check_pad_template (GstPadTemplate * tmpl)
 
   gst_caps_unref (caps);
 
-  for (i = 2; i < num_formats; ++i) {
+  for (i = 2; i < GST_VIDEO_FORMAT_LAST; ++i) {
     if (!formats_supported[i]) {
       const gchar *fmt_str = gst_video_format_to_string ((GstVideoFormat) i);
 
@@ -126,6 +115,13 @@ check_pad_template (GstPadTemplate * tmpl)
         case GST_VIDEO_FORMAT_NV12_64Z32:
         case GST_VIDEO_FORMAT_NV12_4L4:
         case GST_VIDEO_FORMAT_NV12_32L32:
+        case GST_VIDEO_FORMAT_NV12_16L32S:
+        case GST_VIDEO_FORMAT_NV12_8L128:
+        case GST_VIDEO_FORMAT_NV12_10BE_8L128:
+        case GST_VIDEO_FORMAT_NV12_10LE40_4L4:
+        case GST_VIDEO_FORMAT_DMA_DRM:
+        case GST_VIDEO_FORMAT_MT2110T:
+        case GST_VIDEO_FORMAT_MT2110R:
           GST_LOG ("Ignoring lack of support for format %s", fmt_str);
           break;
         default:
@@ -171,7 +167,7 @@ videoscale_get_allowed_caps_for_method (int method)
   GstCaps *caps, **ret;
   GstPad *pad;
   GstStructure *s;
-  gint i, n;
+  gint i, j, n;
 
   scale = gst_element_factory_make ("videoscale", "vscale");
   g_object_set (scale, "method", method, NULL);
@@ -184,11 +180,16 @@ videoscale_get_allowed_caps_for_method (int method)
   n = gst_caps_get_size (caps);
   ret = g_new0 (GstCaps *, n + 1);
 
-  for (i = 0; i < n; i++) {
+
+  for (i = 0, j = 0; i < n; i++) {
+    /* Skip passthrough caps */
+    if (gst_caps_features_is_any (gst_caps_get_features (caps, i)))
+      continue;
     s = gst_caps_get_structure (caps, i);
-    ret[i] = gst_caps_new_empty ();
-    gst_caps_append_structure (ret[i], gst_structure_copy (s));
+    ret[j] = gst_caps_new_empty ();
+    gst_caps_append_structure (ret[j], gst_structure_copy (s));
     GST_LOG ("method %d supports: %" GST_PTR_FORMAT, method, s);
+    j++;
   }
 
   gst_caps_unref (caps);
@@ -963,6 +964,54 @@ GST_START_TEST (test_basetransform_negotiation)
 
 GST_END_TEST;
 
+GST_START_TEST (test_transform_meta)
+{
+  GstHarness *h;
+  GstBuffer *buffer;
+  const guint8 data[16] = { 0 };
+  GstCaps *caps;
+  GstVideoRegionOfInterestMeta *roi_meta;
+  GstRTPSourceMeta *rtp_source_meta;
+  guint32 ssrc = 1234;
+
+  h = gst_harness_new ("videoscale");
+
+  buffer = gst_buffer_new_allocate (NULL, sizeof (data), NULL);
+  gst_buffer_fill (buffer, 0, data, sizeof (data));
+
+  caps = gst_caps_new_empty_simple ("timestamp/test");
+  gst_buffer_add_video_region_of_interest_meta (buffer, "face", 0, 1, 2, 3);
+  rtp_source_meta = gst_buffer_add_rtp_source_meta (buffer, &ssrc, NULL, 0);
+
+  gst_harness_set_sink_caps_str (h,
+      "video/x-raw,width=8,height=8,format=GRAY8");
+  gst_harness_set_src_caps_str (h, "video/x-raw,width=4,height=4,format=GRAY8");
+  fail_unless_equals_int (gst_harness_push (h, buffer), GST_FLOW_OK);
+
+  buffer = gst_harness_pull (h);
+  fail_unless_equals_int (gst_buffer_get_n_meta (buffer,
+          GST_VIDEO_REGION_OF_INTEREST_META_API_TYPE), 1);
+
+  roi_meta = gst_buffer_get_video_region_of_interest_meta_id (buffer, 0);
+  fail_unless (roi_meta != NULL);
+  fail_unless_equals_int (roi_meta->roi_type, g_quark_from_string ("face"));
+  fail_unless_equals_int (roi_meta->x, 0);
+  fail_unless_equals_int (roi_meta->y, 2);
+  fail_unless_equals_int (roi_meta->w, 4);
+  fail_unless_equals_int (roi_meta->h, 6);
+
+  rtp_source_meta = gst_buffer_get_rtp_source_meta (buffer);
+  fail_unless (rtp_source_meta != NULL);
+  fail_unless (rtp_source_meta->ssrc_valid);
+  fail_unless_equals_int (rtp_source_meta->ssrc, ssrc);
+
+  gst_buffer_unref (buffer);
+  gst_caps_unref (caps);
+  gst_harness_teardown (h);
+}
+
+GST_END_TEST;
+
 #endif /* !defined(VSCALE_TEST_GROUP) */
 
 static Suite *
@@ -985,6 +1034,7 @@ videoscale_suite (void)
   tcase_add_test (tc_chain, test_reverse_negotiation);
 #endif
   tcase_add_test (tc_chain, test_basetransform_negotiation);
+  tcase_add_test (tc_chain, test_transform_meta);
 #else
 #if VSCALE_TEST_GROUP == 1
   tcase_add_test (tc_chain, test_downscale_640x480_320x240_method_0);
