@@ -20,7 +20,10 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <gst/check/gstcheck.h>
 
 #include <gst/app/gstappsrc.h>
@@ -62,9 +65,13 @@
 #include <gst/video/gstvideofilter.h>
 #include <gst/video/gstvideosink.h>
 #include <gst/video/colorbalance.h>
+#include <gst/video/videodirection.h>
 #include <gst/video/videoorientation.h>
 #include <gst/video/videooverlay.h>
 #include <gst/video/navigation.h>
+#if defined (TEST_GST_GL_ABI_CHECK)
+#include "gstgl-public-headers.h"
+#endif
 
 /* initial version of the file was generated using:
  * grep -A1 "<STRUCT>" ../../docs/libs/gst-plugins-base-libs-decl.txt | \
@@ -88,6 +95,9 @@
 #elif defined HAVE_CPU_ARM
 # include "struct_arm.h"
 # define HAVE_ABI_SIZES FALSE
+#elif defined HAVE_CPU_AARCH64
+# include "struct_aarch64.h"
+# define HAVE_ABI_SIZES FALSE
 #elif defined HAVE_CPU_PPC
 # include "struct_ppc32.h"
 # define HAVE_ABI_SIZES TRUE
